# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

import os
from typing import Any, cast

import pytest

from reminix import Reminix, AsyncReminix
from tests.utils import assert_matches_type
from reminix.types import Tool, ToolCallResponse
from reminix.pagination import SyncCursor, AsyncCursor

base_url = os.environ.get("TEST_API_BASE_URL", "http://127.0.0.1:4010")


class TestTools:
    parametrize = pytest.mark.parametrize("client", [False, True], indirect=True, ids=["loose", "strict"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_method_retrieve(self, client: Reminix) -> None:
        tool = client.tools.retrieve(
            "x",
        )
        assert_matches_type(Tool, tool, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_raw_response_retrieve(self, client: Reminix) -> None:
        response = client.tools.with_raw_response.retrieve(
            "x",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        tool = response.parse()
        assert_matches_type(Tool, tool, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_streaming_response_retrieve(self, client: Reminix) -> None:
        with client.tools.with_streaming_response.retrieve(
            "x",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            tool = response.parse()
            assert_matches_type(Tool, tool, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_path_params_retrieve(self, client: Reminix) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `name` but received ''"):
            client.tools.with_raw_response.retrieve(
                "",
            )

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_method_list(self, client: Reminix) -> None:
        tool = client.tools.list()
        assert_matches_type(SyncCursor[Tool], tool, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_method_list_with_all_params(self, client: Reminix) -> None:
        tool = client.tools.list(
            cursor="cursor",
            limit=1,
            status="active",
            type="type",
        )
        assert_matches_type(SyncCursor[Tool], tool, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_raw_response_list(self, client: Reminix) -> None:
        response = client.tools.with_raw_response.list()

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        tool = response.parse()
        assert_matches_type(SyncCursor[Tool], tool, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_streaming_response_list(self, client: Reminix) -> None:
        with client.tools.with_streaming_response.list() as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            tool = response.parse()
            assert_matches_type(SyncCursor[Tool], tool, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_method_call(self, client: Reminix) -> None:
        tool = client.tools.call(
            name="x",
            input={
                "name": "bar",
                "greeting": "bar",
            },
        )
        assert_matches_type(ToolCallResponse, tool, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_raw_response_call(self, client: Reminix) -> None:
        response = client.tools.with_raw_response.call(
            name="x",
            input={
                "name": "bar",
                "greeting": "bar",
            },
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        tool = response.parse()
        assert_matches_type(ToolCallResponse, tool, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_streaming_response_call(self, client: Reminix) -> None:
        with client.tools.with_streaming_response.call(
            name="x",
            input={
                "name": "bar",
                "greeting": "bar",
            },
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            tool = response.parse()
            assert_matches_type(ToolCallResponse, tool, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_path_params_call(self, client: Reminix) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `name` but received ''"):
            client.tools.with_raw_response.call(
                name="",
                input={
                    "name": "bar",
                    "greeting": "bar",
                },
            )


class TestAsyncTools:
    parametrize = pytest.mark.parametrize(
        "async_client", [False, True, {"http_client": "aiohttp"}], indirect=True, ids=["loose", "strict", "aiohttp"]
    )

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_method_retrieve(self, async_client: AsyncReminix) -> None:
        tool = await async_client.tools.retrieve(
            "x",
        )
        assert_matches_type(Tool, tool, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_raw_response_retrieve(self, async_client: AsyncReminix) -> None:
        response = await async_client.tools.with_raw_response.retrieve(
            "x",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        tool = await response.parse()
        assert_matches_type(Tool, tool, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_streaming_response_retrieve(self, async_client: AsyncReminix) -> None:
        async with async_client.tools.with_streaming_response.retrieve(
            "x",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            tool = await response.parse()
            assert_matches_type(Tool, tool, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_path_params_retrieve(self, async_client: AsyncReminix) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `name` but received ''"):
            await async_client.tools.with_raw_response.retrieve(
                "",
            )

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_method_list(self, async_client: AsyncReminix) -> None:
        tool = await async_client.tools.list()
        assert_matches_type(AsyncCursor[Tool], tool, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_method_list_with_all_params(self, async_client: AsyncReminix) -> None:
        tool = await async_client.tools.list(
            cursor="cursor",
            limit=1,
            status="active",
            type="type",
        )
        assert_matches_type(AsyncCursor[Tool], tool, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_raw_response_list(self, async_client: AsyncReminix) -> None:
        response = await async_client.tools.with_raw_response.list()

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        tool = await response.parse()
        assert_matches_type(AsyncCursor[Tool], tool, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_streaming_response_list(self, async_client: AsyncReminix) -> None:
        async with async_client.tools.with_streaming_response.list() as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            tool = await response.parse()
            assert_matches_type(AsyncCursor[Tool], tool, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_method_call(self, async_client: AsyncReminix) -> None:
        tool = await async_client.tools.call(
            name="x",
            input={
                "name": "bar",
                "greeting": "bar",
            },
        )
        assert_matches_type(ToolCallResponse, tool, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_raw_response_call(self, async_client: AsyncReminix) -> None:
        response = await async_client.tools.with_raw_response.call(
            name="x",
            input={
                "name": "bar",
                "greeting": "bar",
            },
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        tool = await response.parse()
        assert_matches_type(ToolCallResponse, tool, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_streaming_response_call(self, async_client: AsyncReminix) -> None:
        async with async_client.tools.with_streaming_response.call(
            name="x",
            input={
                "name": "bar",
                "greeting": "bar",
            },
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            tool = await response.parse()
            assert_matches_type(ToolCallResponse, tool, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_path_params_call(self, async_client: AsyncReminix) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `name` but received ''"):
            await async_client.tools.with_raw_response.call(
                name="",
                input={
                    "name": "bar",
                    "greeting": "bar",
                },
            )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from ...._models import BaseModel
from .knowledge_document import KnowledgeDocument

__all__ = ["DocumentProcessResponse"]


class DocumentProcessResponse(BaseModel):
    document: KnowledgeDocument

    message: str



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from datetime import datetime

from pydantic import Field as FieldInfo

from .._models import BaseModel

__all__ = ["ClientTokenCreateResponse"]


class ClientTokenCreateResponse(BaseModel):
    id: str
    """Token ID for management purposes"""

    token: str
    """The client token. Store this securely - it will not be shown again."""

    expires_at: datetime = FieldInfo(alias="expiresAt")
    """ISO 8601 timestamp when the token expires"""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

import os
from typing import TYPE_CHECKING, Any, Mapping
from typing_extensions import Self, override

import httpx

from . import _exceptions
from ._qs import Querystring
from ._types import (
    Omit,
    Timeout,
    NotGiven,
    Transport,
    ProxiesTypes,
    RequestOptions,
    not_given,
)
from ._utils import is_given, get_async_library
from ._compat import cached_property
from ._version import __version__
from ._streaming import Stream as Stream, AsyncStream as AsyncStream
from ._exceptions import ReminixError, APIStatusError
from ._base_client import (
    DEFAULT_MAX_RETRIES,
    SyncAPIClient,
    AsyncAPIClient,
)

if TYPE_CHECKING:
    from .resources import tools, agents, memories, projects, knowledge, client_tokens, conversations, execution_logs
    from .resources.tools import ToolsResource, AsyncToolsResource
    from .resources.agents import AgentsResource, AsyncAgentsResource
    from .resources.memories import MemoriesResource, AsyncMemoriesResource
    from .resources.projects import ProjectsResource, AsyncProjectsResource
    from .resources.client_tokens import ClientTokensResource, AsyncClientTokensResource
    from .resources.conversations import ConversationsResource, AsyncConversationsResource
    from .resources.execution_logs import ExecutionLogsResource, AsyncExecutionLogsResource
    from .resources.knowledge.knowledge import KnowledgeResource, AsyncKnowledgeResource

__all__ = ["Timeout", "Transport", "ProxiesTypes", "RequestOptions", "Reminix", "AsyncReminix", "Client", "AsyncClient"]


class Reminix(SyncAPIClient):
    # client options
    api_key: str

    def __init__(
        self,
        *,
        api_key: str | None = None,
        base_url: str | httpx.URL | None = None,
        timeout: float | Timeout | None | NotGiven = not_given,
        max_retries: int = DEFAULT_MAX_RETRIES,
        default_headers: Mapping[str, str] | None = None,
        default_query: Mapping[str, object] | None = None,
        # Configure a custom httpx client.
        # We provide a `DefaultHttpxClient` class that you can pass to retain the default values we use for `limits`, `timeout` & `follow_redirects`.
        # See the [httpx documentation](https://www.python-httpx.org/api/#client) for more details.
        http_client: httpx.Client | None = None,
        # Enable or disable schema validation for data returned by the API.
        # When enabled an error APIResponseValidationError is raised
        # if the API responds with invalid data for the expected schema.
        #
        # This parameter may be removed or changed in the future.
        # If you rely on this feature, please open a GitHub issue
        # outlining your use-case to help us decide if it should be
        # part of our public interface in the future.
        _strict_response_validation: bool = False,
    ) -> None:
        """Construct a new synchronous Reminix client instance.

        This automatically infers the `api_key` argument from the `REMINIX_API_KEY` environment variable if it is not provided.
        """
        if api_key is None:
            api_key = os.environ.get("REMINIX_API_KEY")
        if api_key is None:
            raise ReminixError(
                "The api_key client option must be set either by passing api_key to the client or by setting the REMINIX_API_KEY environment variable"
            )
        self.api_key = api_key

        if base_url is None:
            base_url = os.environ.get("REMINIX_BASE_URL")
        if base_url is None:
            base_url = f"https://api.reminix.com/v1"

        super().__init__(
            version=__version__,
            base_url=base_url,
            max_retries=max_retries,
            timeout=timeout,
            http_client=http_client,
            custom_headers=default_headers,
            custom_query=default_query,
            _strict_response_validation=_strict_response_validation,
        )

    @cached_property
    def projects(self) -> ProjectsResource:
        from .resources.projects import ProjectsResource

        return ProjectsResource(self)

    @cached_property
    def agents(self) -> AgentsResource:
        from .resources.agents import AgentsResource

        return AgentsResource(self)

    @cached_property
    def tools(self) -> ToolsResource:
        from .resources.tools import ToolsResource

        return ToolsResource(self)

    @cached_property
    def client_tokens(self) -> ClientTokensResource:
        from .resources.client_tokens import ClientTokensResource

        return ClientTokensResource(self)

    @cached_property
    def execution_logs(self) -> ExecutionLogsResource:
        from .resources.execution_logs import ExecutionLogsResource

        return ExecutionLogsResource(self)

    @cached_property
    def conversations(self) -> ConversationsResource:
        from .resources.conversations import ConversationsResource

        return ConversationsResource(self)

    @cached_property
    def memories(self) -> MemoriesResource:
        from .resources.memories import MemoriesResource

        return MemoriesResource(self)

    @cached_property
    def knowledge(self) -> KnowledgeResource:
        from .resources.knowledge import KnowledgeResource

        return KnowledgeResource(self)

    @cached_property
    def with_raw_response(self) -> ReminixWithRawResponse:
        return ReminixWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> ReminixWithStreamedResponse:
        return ReminixWithStreamedResponse(self)

    @property
    @override
    def qs(self) -> Querystring:
        return Querystring(array_format="comma")

    @property
    @override
    def auth_headers(self) -> dict[str, str]:
        api_key = self.api_key
        return {"Authorization": f"Bearer {api_key}"}

    @property
    @override
    def default_headers(self) -> dict[str, str | Omit]:
        return {
            **super().default_headers,
            "X-Stainless-Async": "false",
            "X-Reminix-Source": "sdk",
            **self._custom_headers,
        }

    def copy(
        self,
        *,
        api_key: str | None = None,
        base_url: str | httpx.URL | None = None,
        timeout: float | Timeout | None | NotGiven = not_given,
        http_client: httpx.Client | None = None,
        max_retries: int | NotGiven = not_given,
        default_headers: Mapping[str, str] | None = None,
        set_default_headers: Mapping[str, str] | None = None,
        default_query: Mapping[str, object] | None = None,
        set_default_query: Mapping[str, object] | None = None,
        _extra_kwargs: Mapping[str, Any] = {},
    ) -> Self:
        """
        Create a new client instance re-using the same options given to the current client with optional overriding.
        """
        if default_headers is not None and set_default_headers is not None:
            raise ValueError("The `default_headers` and `set_default_headers` arguments are mutually exclusive")

        if default_query is not None and set_default_query is not None:
            raise ValueError("The `default_query` and `set_default_query` arguments are mutually exclusive")

        headers = self._custom_headers
        if default_headers is not None:
            headers = {**headers, **default_headers}
        elif set_default_headers is not None:
            headers = set_default_headers

        params = self._custom_query
        if default_query is not None:
            params = {**params, **default_query}
        elif set_default_query is not None:
            params = set_default_query

        http_client = http_client or self._client
        return self.__class__(
            api_key=api_key or self.api_key,
            base_url=base_url or self.base_url,
            timeout=self.timeout if isinstance(timeout, NotGiven) else timeout,
            http_client=http_client,
            max_retries=max_retries if is_given(max_retries) else self.max_retries,
            default_headers=headers,
            default_query=params,
            **_extra_kwargs,
        )

    # Alias for `copy` for nicer inline usage, e.g.
    # client.with_options(timeout=10).foo.create(...)
    with_options = copy

    @override
    def _make_status_error(
        self,
        err_msg: str,
        *,
        body: object,
        response: httpx.Response,
    ) -> APIStatusError:
        if response.status_code == 400:
            return _exceptions.BadRequestError(err_msg, response=response, body=body)

        if response.status_code == 401:
            return _exceptions.AuthenticationError(err_msg, response=response, body=body)

        if response.status_code == 403:
            return _exceptions.PermissionDeniedError(err_msg, response=response, body=body)

        if response.status_code == 404:
            return _exceptions.NotFoundError(err_msg, response=response, body=body)

        if response.status_code == 409:
            return _exceptions.ConflictError(err_msg, response=response, body=body)

        if response.status_code == 422:
            return _exceptions.UnprocessableEntityError(err_msg, response=response, body=body)

        if response.status_code == 429:
            return _exceptions.RateLimitError(err_msg, response=response, body=body)

        if response.status_code >= 500:
            return _exceptions.InternalServerError(err_msg, response=response, body=body)
        return APIStatusError(err_msg, response=response, body=body)


class AsyncReminix(AsyncAPIClient):
    # client options
    api_key: str

    def __init__(
        self,
        *,
        api_key: str | None = None,
        base_url: str | httpx.URL | None = None,
        timeout: float | Timeout | None | NotGiven = not_given,
        max_retries: int = DEFAULT_MAX_RETRIES,
        default_headers: Mapping[str, str] | None = None,
        default_query: Mapping[str, object] | None = None,
        # Configure a custom httpx client.
        # We provide a `DefaultAsyncHttpxClient` class that you can pass to retain the default values we use for `limits`, `timeout` & `follow_redirects`.
        # See the [httpx documentation](https://www.python-httpx.org/api/#asyncclient) for more details.
        http_client: httpx.AsyncClient | None = None,
        # Enable or disable schema validation for data returned by the API.
        # When enabled an error APIResponseValidationError is raised
        # if the API responds with invalid data for the expected schema.
        #
        # This parameter may be removed or changed in the future.
        # If you rely on this feature, please open a GitHub issue
        # outlining your use-case to help us decide if it should be
        # part of our public interface in the future.
        _strict_response_validation: bool = False,
    ) -> None:
        """Construct a new async AsyncReminix client instance.

        This automatically infers the `api_key` argument from the `REMINIX_API_KEY` environment variable if it is not provided.
        """
        if api_key is None:
            api_key = os.environ.get("REMINIX_API_KEY")
        if api_key is None:
            raise ReminixError(
                "The api_key client option must be set either by passing api_key to the client or by setting the REMINIX_API_KEY environment variable"
            )
        self.api_key = api_key

        if base_url is None:
            base_url = os.environ.get("REMINIX_BASE_URL")
        if base_url is None:
            base_url = f"https://api.reminix.com/v1"

        super().__init__(
            version=__version__,
            base_url=base_url,
            max_retries=max_retries,
            timeout=timeout,
            http_client=http_client,
            custom_headers=default_headers,
            custom_query=default_query,
            _strict_response_validation=_strict_response_validation,
        )

    @cached_property
    def projects(self) -> AsyncProjectsResource:
        from .resources.projects import AsyncProjectsResource

        return AsyncProjectsResource(self)

    @cached_property
    def agents(self) -> AsyncAgentsResource:
        from .resources.agents import AsyncAgentsResource

        return AsyncAgentsResource(self)

    @cached_property
    def tools(self) -> AsyncToolsResource:
        from .resources.tools import AsyncToolsResource

        return AsyncToolsResource(self)

    @cached_property
    def client_tokens(self) -> AsyncClientTokensResource:
        from .resources.client_tokens import AsyncClientTokensResource

        return AsyncClientTokensResource(self)

    @cached_property
    def execution_logs(self) -> AsyncExecutionLogsResource:
        from .resources.execution_logs import AsyncExecutionLogsResource

        return AsyncExecutionLogsResource(self)

    @cached_property
    def conversations(self) -> AsyncConversationsResource:
        from .resources.conversations import AsyncConversationsResource

        return AsyncConversationsResource(self)

    @cached_property
    def memories(self) -> AsyncMemoriesResource:
        from .resources.memories import AsyncMemoriesResource

        return AsyncMemoriesResource(self)

    @cached_property
    def knowledge(self) -> AsyncKnowledgeResource:
        from .resources.knowledge import AsyncKnowledgeResource

        return AsyncKnowledgeResource(self)

    @cached_property
    def with_raw_response(self) -> AsyncReminixWithRawResponse:
        return AsyncReminixWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncReminixWithStreamedResponse:
        return AsyncReminixWithStreamedResponse(self)

    @property
    @override
    def qs(self) -> Querystring:
        return Querystring(array_format="comma")

    @property
    @override
    def auth_headers(self) -> dict[str, str]:
        api_key = self.api_key
        return {"Authorization": f"Bearer {api_key}"}

    @property
    @override
    def default_headers(self) -> dict[str, str | Omit]:
        return {
            **super().default_headers,
            "X-Stainless-Async": f"async:{get_async_library()}",
            "X-Reminix-Source": "sdk",
            **self._custom_headers,
        }

    def copy(
        self,
        *,
        api_key: str | None = None,
        base_url: str | httpx.URL | None = None,
        timeout: float | Timeout | None | NotGiven = not_given,
        http_client: httpx.AsyncClient | None = None,
        max_retries: int | NotGiven = not_given,
        default_headers: Mapping[str, str] | None = None,
        set_default_headers: Mapping[str, str] | None = None,
        default_query: Mapping[str, object] | None = None,
        set_default_query: Mapping[str, object] | None = None,
        _extra_kwargs: Mapping[str, Any] = {},
    ) -> Self:
        """
        Create a new client instance re-using the same options given to the current client with optional overriding.
        """
        if default_headers is not None and set_default_headers is not None:
            raise ValueError("The `default_headers` and `set_default_headers` arguments are mutually exclusive")

        if default_query is not None and set_default_query is not None:
            raise ValueError("The `default_query` and `set_default_query` arguments are mutually exclusive")

        headers = self._custom_headers
        if default_headers is not None:
            headers = {**headers, **default_headers}
        elif set_default_headers is not None:
            headers = set_default_headers

        params = self._custom_query
        if default_query is not None:
            params = {**params, **default_query}
        elif set_default_query is not None:
            params = set_default_query

        http_client = http_client or self._client
        return self.__class__(
            api_key=api_key or self.api_key,
            base_url=base_url or self.base_url,
            timeout=self.timeout if isinstance(timeout, NotGiven) else timeout,
            http_client=http_client,
            max_retries=max_retries if is_given(max_retries) else self.max_retries,
            default_headers=headers,
            default_query=params,
            **_extra_kwargs,
        )

    # Alias for `copy` for nicer inline usage, e.g.
    # client.with_options(timeout=10).foo.create(...)
    with_options = copy

    @override
    def _make_status_error(
        self,
        err_msg: str,
        *,
        body: object,
        response: httpx.Response,
    ) -> APIStatusError:
        if response.status_code == 400:
            return _exceptions.BadRequestError(err_msg, response=response, body=body)

        if response.status_code == 401:
            return _exceptions.AuthenticationError(err_msg, response=response, body=body)

        if response.status_code == 403:
            return _exceptions.PermissionDeniedError(err_msg, response=response, body=body)

        if response.status_code == 404:
            return _exceptions.NotFoundError(err_msg, response=response, body=body)

        if response.status_code == 409:
            return _exceptions.ConflictError(err_msg, response=response, body=body)

        if response.status_code == 422:
            return _exceptions.UnprocessableEntityError(err_msg, response=response, body=body)

        if response.status_code == 429:
            return _exceptions.RateLimitError(err_msg, response=response, body=body)

        if response.status_code >= 500:
            return _exceptions.InternalServerError(err_msg, response=response, body=body)
        return APIStatusError(err_msg, response=response, body=body)


class ReminixWithRawResponse:
    _client: Reminix

    def __init__(self, client: Reminix) -> None:
        self._client = client

    @cached_property
    def projects(self) -> projects.ProjectsResourceWithRawResponse:
        from .resources.projects import ProjectsResourceWithRawResponse

        return ProjectsResourceWithRawResponse(self._client.projects)

    @cached_property
    def agents(self) -> agents.AgentsResourceWithRawResponse:
        from .resources.agents import AgentsResourceWithRawResponse

        return AgentsResourceWithRawResponse(self._client.agents)

    @cached_property
    def tools(self) -> tools.ToolsResourceWithRawResponse:
        from .resources.tools import ToolsResourceWithRawResponse

        return ToolsResourceWithRawResponse(self._client.tools)

    @cached_property
    def client_tokens(self) -> client_tokens.ClientTokensResourceWithRawResponse:
        from .resources.client_tokens import ClientTokensResourceWithRawResponse

        return ClientTokensResourceWithRawResponse(self._client.client_tokens)

    @cached_property
    def execution_logs(self) -> execution_logs.ExecutionLogsResourceWithRawResponse:
        from .resources.execution_logs import ExecutionLogsResourceWithRawResponse

        return ExecutionLogsResourceWithRawResponse(self._client.execution_logs)

    @cached_property
    def conversations(self) -> conversations.ConversationsResourceWithRawResponse:
        from .resources.conversations import ConversationsResourceWithRawResponse

        return ConversationsResourceWithRawResponse(self._client.conversations)

    @cached_property
    def memories(self) -> memories.MemoriesResourceWithRawResponse:
        from .resources.memories import MemoriesResourceWithRawResponse

        return MemoriesResourceWithRawResponse(self._client.memories)

    @cached_property
    def knowledge(self) -> knowledge.KnowledgeResourceWithRawResponse:
        from .resources.knowledge import KnowledgeResourceWithRawResponse

        return KnowledgeResourceWithRawResponse(self._client.knowledge)


class AsyncReminixWithRawResponse:
    _client: AsyncReminix

    def __init__(self, client: AsyncReminix) -> None:
        self._client = client

    @cached_property
    def projects(self) -> projects.AsyncProjectsResourceWithRawResponse:
        from .resources.projects import AsyncProjectsResourceWithRawResponse

        return AsyncProjectsResourceWithRawResponse(self._client.projects)

    @cached_property
    def agents(self) -> agents.AsyncAgentsResourceWithRawResponse:
        from .resources.agents import AsyncAgentsResourceWithRawResponse

        return AsyncAgentsResourceWithRawResponse(self._client.agents)

    @cached_property
    def tools(self) -> tools.AsyncToolsResourceWithRawResponse:
        from .resources.tools import AsyncToolsResourceWithRawResponse

        return AsyncToolsResourceWithRawResponse(self._client.tools)

    @cached_property
    def client_tokens(self) -> client_tokens.AsyncClientTokensResourceWithRawResponse:
        from .resources.client_tokens import AsyncClientTokensResourceWithRawResponse

        return AsyncClientTokensResourceWithRawResponse(self._client.client_tokens)

    @cached_property
    def execution_logs(self) -> execution_logs.AsyncExecutionLogsResourceWithRawResponse:
        from .resources.execution_logs import AsyncExecutionLogsResourceWithRawResponse

        return AsyncExecutionLogsResourceWithRawResponse(self._client.execution_logs)

    @cached_property
    def conversations(self) -> conversations.AsyncConversationsResourceWithRawResponse:
        from .resources.conversations import AsyncConversationsResourceWithRawResponse

        return AsyncConversationsResourceWithRawResponse(self._client.conversations)

    @cached_property
    def memories(self) -> memories.AsyncMemoriesResourceWithRawResponse:
        from .resources.memories import AsyncMemoriesResourceWithRawResponse

        return AsyncMemoriesResourceWithRawResponse(self._client.memories)

    @cached_property
    def knowledge(self) -> knowledge.AsyncKnowledgeResourceWithRawResponse:
        from .resources.knowledge import AsyncKnowledgeResourceWithRawResponse

        return AsyncKnowledgeResourceWithRawResponse(self._client.knowledge)


class ReminixWithStreamedResponse:
    _client: Reminix

    def __init__(self, client: Reminix) -> None:
        self._client = client

    @cached_property
    def projects(self) -> projects.ProjectsResourceWithStreamingResponse:
        from .resources.projects import ProjectsResourceWithStreamingResponse

        return ProjectsResourceWithStreamingResponse(self._client.projects)

    @cached_property
    def agents(self) -> agents.AgentsResourceWithStreamingResponse:
        from .resources.agents import AgentsResourceWithStreamingResponse

        return AgentsResourceWithStreamingResponse(self._client.agents)

    @cached_property
    def tools(self) -> tools.ToolsResourceWithStreamingResponse:
        from .resources.tools import ToolsResourceWithStreamingResponse

        return ToolsResourceWithStreamingResponse(self._client.tools)

    @cached_property
    def client_tokens(self) -> client_tokens.ClientTokensResourceWithStreamingResponse:
        from .resources.client_tokens import ClientTokensResourceWithStreamingResponse

        return ClientTokensResourceWithStreamingResponse(self._client.client_tokens)

    @cached_property
    def execution_logs(self) -> execution_logs.ExecutionLogsResourceWithStreamingResponse:
        from .resources.execution_logs import ExecutionLogsResourceWithStreamingResponse

        return ExecutionLogsResourceWithStreamingResponse(self._client.execution_logs)

    @cached_property
    def conversations(self) -> conversations.ConversationsResourceWithStreamingResponse:
        from .resources.conversations import ConversationsResourceWithStreamingResponse

        return ConversationsResourceWithStreamingResponse(self._client.conversations)

    @cached_property
    def memories(self) -> memories.MemoriesResourceWithStreamingResponse:
        from .resources.memories import MemoriesResourceWithStreamingResponse

        return MemoriesResourceWithStreamingResponse(self._client.memories)

    @cached_property
    def knowledge(self) -> knowledge.KnowledgeResourceWithStreamingResponse:
        from .resources.knowledge import KnowledgeResourceWithStreamingResponse

        return KnowledgeResourceWithStreamingResponse(self._client.knowledge)


class AsyncReminixWithStreamedResponse:
    _client: AsyncReminix

    def __init__(self, client: AsyncReminix) -> None:
        self._client = client

    @cached_property
    def projects(self) -> projects.AsyncProjectsResourceWithStreamingResponse:
        from .resources.projects import AsyncProjectsResourceWithStreamingResponse

        return AsyncProjectsResourceWithStreamingResponse(self._client.projects)

    @cached_property
    def agents(self) -> agents.AsyncAgentsResourceWithStreamingResponse:
        from .resources.agents import AsyncAgentsResourceWithStreamingResponse

        return AsyncAgentsResourceWithStreamingResponse(self._client.agents)

    @cached_property
    def tools(self) -> tools.AsyncToolsResourceWithStreamingResponse:
        from .resources.tools import AsyncToolsResourceWithStreamingResponse

        return AsyncToolsResourceWithStreamingResponse(self._client.tools)

    @cached_property
    def client_tokens(self) -> client_tokens.AsyncClientTokensResourceWithStreamingResponse:
        from .resources.client_tokens import AsyncClientTokensResourceWithStreamingResponse

        return AsyncClientTokensResourceWithStreamingResponse(self._client.client_tokens)

    @cached_property
    def execution_logs(self) -> execution_logs.AsyncExecutionLogsResourceWithStreamingResponse:
        from .resources.execution_logs import AsyncExecutionLogsResourceWithStreamingResponse

        return AsyncExecutionLogsResourceWithStreamingResponse(self._client.execution_logs)

    @cached_property
    def conversations(self) -> conversations.AsyncConversationsResourceWithStreamingResponse:
        from .resources.conversations import AsyncConversationsResourceWithStreamingResponse

        return AsyncConversationsResourceWithStreamingResponse(self._client.conversations)

    @cached_property
    def memories(self) -> memories.AsyncMemoriesResourceWithStreamingResponse:
        from .resources.memories import AsyncMemoriesResourceWithStreamingResponse

        return AsyncMemoriesResourceWithStreamingResponse(self._client.memories)

    @cached_property
    def knowledge(self) -> knowledge.AsyncKnowledgeResourceWithStreamingResponse:
        from .resources.knowledge import AsyncKnowledgeResourceWithStreamingResponse

        return AsyncKnowledgeResourceWithStreamingResponse(self._client.knowledge)


Client = Reminix

AsyncClient = AsyncReminix



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from .._models import BaseModel

__all__ = ["AgentMemoryConfig"]


class AgentMemoryConfig(BaseModel):
    """Memory feature configuration"""

    enabled: bool
    """Whether memory is enabled for this agent"""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import List, Optional
from typing_extensions import Literal

from pydantic import Field as FieldInfo

from .._models import BaseModel
from .agent_memory_config import AgentMemoryConfig
from .agent_knowledge_base_config import AgentKnowledgeBaseConfig

__all__ = ["AgentConfig"]


class AgentConfig(BaseModel):
    """Agent configuration (for managed agents)"""

    model: str
    """Model identifier (e.g., "gpt-4o", "claude-sonnet-4-20250514")"""

    provider: Literal["openai", "anthropic"]
    """LLM provider"""

    system_prompt: str = FieldInfo(alias="systemPrompt")
    """System prompt for the agent"""

    tools: List[str]
    """List of tools available to the agent"""

    knowledge_base: Optional[AgentKnowledgeBaseConfig] = FieldInfo(alias="knowledgeBase", default=None)
    """Knowledge base feature configuration"""

    max_iterations: Optional[float] = FieldInfo(alias="maxIterations", default=None)
    """Maximum tool call iterations"""

    memory: Optional[AgentMemoryConfig] = None
    """Memory feature configuration"""

    require_approval: Optional[bool] = FieldInfo(alias="requireApproval", default=None)
    """Whether to require approval for tool calls"""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Required, Annotated, TypedDict

from .._types import SequenceNotStr
from .._utils import PropertyInfo

__all__ = ["KnowledgeSearchParams"]


class KnowledgeSearchParams(TypedDict, total=False):
    collection_ids: Required[Annotated[SequenceNotStr[str], PropertyInfo(alias="collectionIds")]]
    """Collection IDs to search"""

    query: Required[str]
    """Natural language search query"""

    limit: float
    """Maximum number of results"""

    threshold: float
    """Minimum similarity score (0-1)"""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from datetime import datetime

from pydantic import Field as FieldInfo

from .._models import BaseModel

__all__ = ["Project", "Organization"]


class Organization(BaseModel):
    """Organization that owns this project"""

    id: str
    """Unique identifier for the organization"""

    name: str
    """Human-readable name of the organization"""

    slug: str
    """URL-friendly identifier for the organization"""


class Project(BaseModel):
    id: str
    """Unique identifier for the project"""

    created_at: datetime = FieldInfo(alias="createdAt")
    """ISO 8601 timestamp when the project was created"""

    name: str
    """Human-readable name of the project"""

    organization: Organization
    """Organization that owns this project"""

    organization_id: str = FieldInfo(alias="organizationId")
    """ID of the organization that owns this project"""

    slug: str
    """URL-friendly identifier for the project"""

    updated_at: datetime = FieldInfo(alias="updatedAt")
    """ISO 8601 timestamp when the project was last updated"""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Dict, Union, Iterable, Optional
from typing_extensions import Literal, Required, TypeAlias, TypedDict

from .chat_message_param import ChatMessageParam

__all__ = ["AgentChatParamsBase", "Context", "AgentChatParamsNonStreaming", "AgentChatParamsStreaming"]


class AgentChatParamsBase(TypedDict, total=False):
    messages: Required[Iterable[ChatMessageParam]]
    """Array of chat messages"""

    context: Context
    """Optional context for the agent execution"""

    conversation_id: str
    """Conversation ID to continue an existing conversation"""


class ContextTyped(TypedDict, total=False):
    """Optional context for the agent execution"""

    identity: Dict[str, Optional[object]]
    """Identity fields for conversation scoping (e.g., user_id, tenant_id)"""


Context: TypeAlias = Union[ContextTyped, Dict[str, Optional[object]]]


class AgentChatParamsNonStreaming(AgentChatParamsBase, total=False):
    stream: Literal[False]
    """Enable streaming response"""


class AgentChatParamsStreaming(AgentChatParamsBase):
    stream: Required[Literal[True]]
    """Enable streaming response"""


AgentChatParams = Union[AgentChatParamsNonStreaming, AgentChatParamsStreaming]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from .tools import (
    ToolsResource,
    AsyncToolsResource,
    ToolsResourceWithRawResponse,
    AsyncToolsResourceWithRawResponse,
    ToolsResourceWithStreamingResponse,
    AsyncToolsResourceWithStreamingResponse,
)
from .agents import (
    AgentsResource,
    AsyncAgentsResource,
    AgentsResourceWithRawResponse,
    AsyncAgentsResourceWithRawResponse,
    AgentsResourceWithStreamingResponse,
    AsyncAgentsResourceWithStreamingResponse,
)
from .memories import (
    MemoriesResource,
    AsyncMemoriesResource,
    MemoriesResourceWithRawResponse,
    AsyncMemoriesResourceWithRawResponse,
    MemoriesResourceWithStreamingResponse,
    AsyncMemoriesResourceWithStreamingResponse,
)
from .projects import (
    ProjectsResource,
    AsyncProjectsResource,
    ProjectsResourceWithRawResponse,
    AsyncProjectsResourceWithRawResponse,
    ProjectsResourceWithStreamingResponse,
    AsyncProjectsResourceWithStreamingResponse,
)
from .knowledge import (
    KnowledgeResource,
    AsyncKnowledgeResource,
    KnowledgeResourceWithRawResponse,
    AsyncKnowledgeResourceWithRawResponse,
    KnowledgeResourceWithStreamingResponse,
    AsyncKnowledgeResourceWithStreamingResponse,
)
from .client_tokens import (
    ClientTokensResource,
    AsyncClientTokensResource,
    ClientTokensResourceWithRawResponse,
    AsyncClientTokensResourceWithRawResponse,
    ClientTokensResourceWithStreamingResponse,
    AsyncClientTokensResourceWithStreamingResponse,
)
from .conversations import (
    ConversationsResource,
    AsyncConversationsResource,
    ConversationsResourceWithRawResponse,
    AsyncConversationsResourceWithRawResponse,
    ConversationsResourceWithStreamingResponse,
    AsyncConversationsResourceWithStreamingResponse,
)
from .execution_logs import (
    ExecutionLogsResource,
    AsyncExecutionLogsResource,
    ExecutionLogsResourceWithRawResponse,
    AsyncExecutionLogsResourceWithRawResponse,
    ExecutionLogsResourceWithStreamingResponse,
    AsyncExecutionLogsResourceWithStreamingResponse,
)

__all__ = [
    "ProjectsResource",
    "AsyncProjectsResource",
    "ProjectsResourceWithRawResponse",
    "AsyncProjectsResourceWithRawResponse",
    "ProjectsResourceWithStreamingResponse",
    "AsyncProjectsResourceWithStreamingResponse",
    "AgentsResource",
    "AsyncAgentsResource",
    "AgentsResourceWithRawResponse",
    "AsyncAgentsResourceWithRawResponse",
    "AgentsResourceWithStreamingResponse",
    "AsyncAgentsResourceWithStreamingResponse",
    "ToolsResource",
    "AsyncToolsResource",
    "ToolsResourceWithRawResponse",
    "AsyncToolsResourceWithRawResponse",
    "ToolsResourceWithStreamingResponse",
    "AsyncToolsResourceWithStreamingResponse",
    "ClientTokensResource",
    "AsyncClientTokensResource",
    "ClientTokensResourceWithRawResponse",
    "AsyncClientTokensResourceWithRawResponse",
    "ClientTokensResourceWithStreamingResponse",
    "AsyncClientTokensResourceWithStreamingResponse",
    "ExecutionLogsResource",
    "AsyncExecutionLogsResource",
    "ExecutionLogsResourceWithRawResponse",
    "AsyncExecutionLogsResourceWithRawResponse",
    "ExecutionLogsResourceWithStreamingResponse",
    "AsyncExecutionLogsResourceWithStreamingResponse",
    "ConversationsResource",
    "AsyncConversationsResource",
    "ConversationsResourceWithRawResponse",
    "AsyncConversationsResourceWithRawResponse",
    "ConversationsResourceWithStreamingResponse",
    "AsyncConversationsResourceWithStreamingResponse",
    "MemoriesResource",
    "AsyncMemoriesResource",
    "MemoriesResourceWithRawResponse",
    "AsyncMemoriesResourceWithRawResponse",
    "MemoriesResourceWithStreamingResponse",
    "AsyncMemoriesResourceWithStreamingResponse",
    "KnowledgeResource",
    "AsyncKnowledgeResource",
    "KnowledgeResourceWithRawResponse",
    "AsyncKnowledgeResourceWithRawResponse",
    "KnowledgeResourceWithStreamingResponse",
    "AsyncKnowledgeResourceWithStreamingResponse",
]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from .knowledge_collection import KnowledgeCollection as KnowledgeCollection
from .collection_list_params import CollectionListParams as CollectionListParams
from .collection_create_params import CollectionCreateParams as CollectionCreateParams
from .collection_update_params import CollectionUpdateParams as CollectionUpdateParams



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from .documents import (
    DocumentsResource,
    AsyncDocumentsResource,
    DocumentsResourceWithRawResponse,
    AsyncDocumentsResourceWithRawResponse,
    DocumentsResourceWithStreamingResponse,
    AsyncDocumentsResourceWithStreamingResponse,
)
from .collections import (
    CollectionsResource,
    AsyncCollectionsResource,
    CollectionsResourceWithRawResponse,
    AsyncCollectionsResourceWithRawResponse,
    CollectionsResourceWithStreamingResponse,
    AsyncCollectionsResourceWithStreamingResponse,
)

__all__ = [
    "DocumentsResource",
    "AsyncDocumentsResource",
    "DocumentsResourceWithRawResponse",
    "AsyncDocumentsResourceWithRawResponse",
    "DocumentsResourceWithStreamingResponse",
    "AsyncDocumentsResourceWithStreamingResponse",
    "CollectionsResource",
    "AsyncCollectionsResource",
    "CollectionsResourceWithRawResponse",
    "AsyncCollectionsResourceWithRawResponse",
    "CollectionsResourceWithStreamingResponse",
    "AsyncCollectionsResourceWithStreamingResponse",
]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Literal

import httpx

from ...._types import Body, Omit, Query, Headers, NoneType, NotGiven, omit, not_given
from ...._utils import maybe_transform, async_maybe_transform
from ...._compat import cached_property
from ...._resource import SyncAPIResource, AsyncAPIResource
from ...._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from ....pagination import SyncCursor, AsyncCursor
from ...._base_client import AsyncPaginator, make_request_options
from ....types.knowledge.collections import document_list_params, document_upload_params, document_process_params
from ....types.knowledge.collections.knowledge_document import KnowledgeDocument
from ....types.knowledge.collections.document_upload_response import DocumentUploadResponse
from ....types.knowledge.collections.document_process_response import DocumentProcessResponse

__all__ = ["DocumentsResource", "AsyncDocumentsResource"]


class DocumentsResource(SyncAPIResource):
    @cached_property
    def with_raw_response(self) -> DocumentsResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/reminix-ai/reminix-python#accessing-raw-response-data-eg-headers
        """
        return DocumentsResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> DocumentsResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/reminix-ai/reminix-python#with_streaming_response
        """
        return DocumentsResourceWithStreamingResponse(self)

    def retrieve(
        self,
        document_id: str,
        *,
        collection_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> KnowledgeDocument:
        """
        Get details of a document.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not collection_id:
            raise ValueError(f"Expected a non-empty value for `collection_id` but received {collection_id!r}")
        if not document_id:
            raise ValueError(f"Expected a non-empty value for `document_id` but received {document_id!r}")
        return self._get(
            f"/knowledge/collections/{collection_id}/documents/{document_id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=KnowledgeDocument,
        )

    def list(
        self,
        id: str,
        *,
        cursor: str | Omit = omit,
        limit: float | Omit = omit,
        status: Literal["pending", "processing", "ready", "failed"] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> SyncCursor[KnowledgeDocument]:
        """
        List all documents in a knowledge collection.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        return self._get_api_list(
            f"/knowledge/collections/{id}/documents",
            page=SyncCursor[KnowledgeDocument],
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "cursor": cursor,
                        "limit": limit,
                        "status": status,
                    },
                    document_list_params.DocumentListParams,
                ),
            ),
            model=KnowledgeDocument,
        )

    def delete(
        self,
        document_id: str,
        *,
        collection_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> None:
        """
        Delete a document and its associated data.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not collection_id:
            raise ValueError(f"Expected a non-empty value for `collection_id` but received {collection_id!r}")
        if not document_id:
            raise ValueError(f"Expected a non-empty value for `document_id` but received {document_id!r}")
        extra_headers = {"Accept": "*/*", **(extra_headers or {})}
        return self._delete(
            f"/knowledge/collections/{collection_id}/documents/{document_id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=NoneType,
        )

    def process(
        self,
        document_id: str,
        *,
        collection_id: str,
        prefer_unstructured: bool | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> DocumentProcessResponse:
        """
        Trigger processing for a document after upload.

        This extracts text from the document, splits it into chunks, generates
        embeddings, and stores them for search.

        The document status will transition: pending → processing → ready (or failed).

        Args:
          prefer_unstructured: Use Unstructured.io for extraction if available

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not collection_id:
            raise ValueError(f"Expected a non-empty value for `collection_id` but received {collection_id!r}")
        if not document_id:
            raise ValueError(f"Expected a non-empty value for `document_id` but received {document_id!r}")
        return self._post(
            f"/knowledge/collections/{collection_id}/documents/{document_id}/process",
            body=maybe_transform(
                {"prefer_unstructured": prefer_unstructured}, document_process_params.DocumentProcessParams
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=DocumentProcessResponse,
        )

    def upload(
        self,
        id: str,
        *,
        mime_type: str,
        name: str,
        size: float | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> DocumentUploadResponse:
        """
        Create a document record and get a presigned URL for upload.

        After receiving the response, upload the file to the presigned URL using a PUT
        request. The document will be processed automatically after upload.

        Args:
          mime_type: MIME type of the document

          name: Document name (filename)

          size: File size in bytes

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        return self._post(
            f"/knowledge/collections/{id}/documents",
            body=maybe_transform(
                {
                    "mime_type": mime_type,
                    "name": name,
                    "size": size,
                },
                document_upload_params.DocumentUploadParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=DocumentUploadResponse,
        )


class AsyncDocumentsResource(AsyncAPIResource):
    @cached_property
    def with_raw_response(self) -> AsyncDocumentsResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/reminix-ai/reminix-python#accessing-raw-response-data-eg-headers
        """
        return AsyncDocumentsResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncDocumentsResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/reminix-ai/reminix-python#with_streaming_response
        """
        return AsyncDocumentsResourceWithStreamingResponse(self)

    async def retrieve(
        self,
        document_id: str,
        *,
        collection_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> KnowledgeDocument:
        """
        Get details of a document.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not collection_id:
            raise ValueError(f"Expected a non-empty value for `collection_id` but received {collection_id!r}")
        if not document_id:
            raise ValueError(f"Expected a non-empty value for `document_id` but received {document_id!r}")
        return await self._get(
            f"/knowledge/collections/{collection_id}/documents/{document_id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=KnowledgeDocument,
        )

    def list(
        self,
        id: str,
        *,
        cursor: str | Omit = omit,
        limit: float | Omit = omit,
        status: Literal["pending", "processing", "ready", "failed"] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> AsyncPaginator[KnowledgeDocument, AsyncCursor[KnowledgeDocument]]:
        """
        List all documents in a knowledge collection.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        return self._get_api_list(
            f"/knowledge/collections/{id}/documents",
            page=AsyncCursor[KnowledgeDocument],
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "cursor": cursor,
                        "limit": limit,
                        "status": status,
                    },
                    document_list_params.DocumentListParams,
                ),
            ),
            model=KnowledgeDocument,
        )

    async def delete(
        self,
        document_id: str,
        *,
        collection_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> None:
        """
        Delete a document and its associated data.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not collection_id:
            raise ValueError(f"Expected a non-empty value for `collection_id` but received {collection_id!r}")
        if not document_id:
            raise ValueError(f"Expected a non-empty value for `document_id` but received {document_id!r}")
        extra_headers = {"Accept": "*/*", **(extra_headers or {})}
        return await self._delete(
            f"/knowledge/collections/{collection_id}/documents/{document_id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=NoneType,
        )

    async def process(
        self,
        document_id: str,
        *,
        collection_id: str,
        prefer_unstructured: bool | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> DocumentProcessResponse:
        """
        Trigger processing for a document after upload.

        This extracts text from the document, splits it into chunks, generates
        embeddings, and stores them for search.

        The document status will transition: pending → processing → ready (or failed).

        Args:
          prefer_unstructured: Use Unstructured.io for extraction if available

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not collection_id:
            raise ValueError(f"Expected a non-empty value for `collection_id` but received {collection_id!r}")
        if not document_id:
            raise ValueError(f"Expected a non-empty value for `document_id` but received {document_id!r}")
        return await self._post(
            f"/knowledge/collections/{collection_id}/documents/{document_id}/process",
            body=await async_maybe_transform(
                {"prefer_unstructured": prefer_unstructured}, document_process_params.DocumentProcessParams
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=DocumentProcessResponse,
        )

    async def upload(
        self,
        id: str,
        *,
        mime_type: str,
        name: str,
        size: float | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> DocumentUploadResponse:
        """
        Create a document record and get a presigned URL for upload.

        After receiving the response, upload the file to the presigned URL using a PUT
        request. The document will be processed automatically after upload.

        Args:
          mime_type: MIME type of the document

          name: Document name (filename)

          size: File size in bytes

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        return await self._post(
            f"/knowledge/collections/{id}/documents",
            body=await async_maybe_transform(
                {
                    "mime_type": mime_type,
                    "name": name,
                    "size": size,
                },
                document_upload_params.DocumentUploadParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=DocumentUploadResponse,
        )


class DocumentsResourceWithRawResponse:
    def __init__(self, documents: DocumentsResource) -> None:
        self._documents = documents

        self.retrieve = to_raw_response_wrapper(
            documents.retrieve,
        )
        self.list = to_raw_response_wrapper(
            documents.list,
        )
        self.delete = to_raw_response_wrapper(
            documents.delete,
        )
        self.process = to_raw_response_wrapper(
            documents.process,
        )
        self.upload = to_raw_response_wrapper(
            documents.upload,
        )


class AsyncDocumentsResourceWithRawResponse:
    def __init__(self, documents: AsyncDocumentsResource) -> None:
        self._documents = documents

        self.retrieve = async_to_raw_response_wrapper(
            documents.retrieve,
        )
        self.list = async_to_raw_response_wrapper(
            documents.list,
        )
        self.delete = async_to_raw_response_wrapper(
            documents.delete,
        )
        self.process = async_to_raw_response_wrapper(
            documents.process,
        )
        self.upload = async_to_raw_response_wrapper(
            documents.upload,
        )


class DocumentsResourceWithStreamingResponse:
    def __init__(self, documents: DocumentsResource) -> None:
        self._documents = documents

        self.retrieve = to_streamed_response_wrapper(
            documents.retrieve,
        )
        self.list = to_streamed_response_wrapper(
            documents.list,
        )
        self.delete = to_streamed_response_wrapper(
            documents.delete,
        )
        self.process = to_streamed_response_wrapper(
            documents.process,
        )
        self.upload = to_streamed_response_wrapper(
            documents.upload,
        )


class AsyncDocumentsResourceWithStreamingResponse:
    def __init__(self, documents: AsyncDocumentsResource) -> None:
        self._documents = documents

        self.retrieve = async_to_streamed_response_wrapper(
            documents.retrieve,
        )
        self.list = async_to_streamed_response_wrapper(
            documents.list,
        )
        self.delete = async_to_streamed_response_wrapper(
            documents.delete,
        )
        self.process = async_to_streamed_response_wrapper(
            documents.process,
        )
        self.upload = async_to_streamed_response_wrapper(
            documents.upload,
        )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Literal, TypedDict

__all__ = ["ExecutionLogListParams"]


class ExecutionLogListParams(TypedDict, total=False):
    cursor: str
    """Cursor for pagination"""

    limit: float
    """Number of logs to return"""

    name: str
    """Filter by agent or tool name"""

    source: Literal["api", "cli", "dashboard", "widget", "sdk"]
    """Filter by request source"""

    status: Literal["success", "error", "timeout"]
    """Filter by execution status"""

    type: Literal["agent_invoke", "agent_chat", "tool_call"]
    """Filter by execution type"""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Optional

from .._models import BaseModel

__all__ = ["ToolCallResponse"]


class ToolCallResponse(BaseModel):
    output: Optional[object] = None
    """Output from the tool execution."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import List

from pydantic import Field as FieldInfo

from .._models import BaseModel

__all__ = ["AgentKnowledgeBaseConfig"]


class AgentKnowledgeBaseConfig(BaseModel):
    """Knowledge base feature configuration"""

    collection_ids: List[str] = FieldInfo(alias="collectionIds")
    """Collection IDs to search"""

    enabled: bool
    """Whether knowledge base is enabled for this agent"""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import List, Optional

from .._models import BaseModel
from .chat_message import ChatMessage

__all__ = ["AgentChatResponse"]


class AgentChatResponse(BaseModel):
    messages: List[ChatMessage]
    """Array of assistant response messages"""

    conversation_id: Optional[str] = None
    """Conversation ID (present when conversation persistence is enabled)"""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from .._models import BaseModel

__all__ = ["MemoryDeleteAllResponse"]


class MemoryDeleteAllResponse(BaseModel):
    deleted: float
    """Number of memories deleted"""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Literal, TypedDict

__all__ = ["DocumentListParams"]


class DocumentListParams(TypedDict, total=False):
    cursor: str

    limit: float

    status: Literal["pending", "processing", "ready", "failed"]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from .knowledge import (
    KnowledgeResource,
    AsyncKnowledgeResource,
    KnowledgeResourceWithRawResponse,
    AsyncKnowledgeResourceWithRawResponse,
    KnowledgeResourceWithStreamingResponse,
    AsyncKnowledgeResourceWithStreamingResponse,
)
from .collections import (
    CollectionsResource,
    AsyncCollectionsResource,
    CollectionsResourceWithRawResponse,
    AsyncCollectionsResourceWithRawResponse,
    CollectionsResourceWithStreamingResponse,
    AsyncCollectionsResourceWithStreamingResponse,
)

__all__ = [
    "CollectionsResource",
    "AsyncCollectionsResource",
    "CollectionsResourceWithRawResponse",
    "AsyncCollectionsResourceWithRawResponse",
    "CollectionsResourceWithStreamingResponse",
    "AsyncCollectionsResourceWithStreamingResponse",
    "KnowledgeResource",
    "AsyncKnowledgeResource",
    "KnowledgeResourceWithRawResponse",
    "AsyncKnowledgeResourceWithRawResponse",
    "KnowledgeResourceWithStreamingResponse",
    "AsyncKnowledgeResourceWithStreamingResponse",
]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import TypedDict

__all__ = ["ConversationListParams"]


class ConversationListParams(TypedDict, total=False):
    agent_name: str
    """Filter by agent name"""

    cursor: str
    """Cursor for pagination"""

    limit: float
    """Number of conversations to return"""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Literal, TypedDict

__all__ = ["ToolListParams"]


class ToolListParams(TypedDict, total=False):
    cursor: str
    """Cursor for pagination"""

    limit: float
    """Number of tools to return"""

    status: Literal["active", "inactive"]
    """Filter by tool status"""

    type: str
    """Filter by tool type (python, typescript)"""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import TYPE_CHECKING, Dict, List, Optional
from typing_extensions import Literal

from pydantic import Field as FieldInfo

from .._models import BaseModel

__all__ = ["Tool", "Input", "Output"]


class Input(BaseModel):
    """JSON Schema for agent input"""

    default: Optional[object] = None
    """Default value for the property"""

    description: Optional[str] = None
    """Description of the schema"""

    enum: Optional[List[Optional[object]]] = None
    """Enumeration of allowed values"""

    items: Optional[object] = None
    """Schema for array items"""

    properties: Optional[Dict[str, Optional[object]]] = None
    """Property definitions for object types"""

    required: Optional[List[str]] = None
    """List of required property names"""

    title: Optional[str] = None
    """Human-readable title"""

    type: Optional[str] = None
    """JSON Schema type (e.g., "object", "string", "array")"""

    if TYPE_CHECKING:
        # Some versions of Pydantic <2.8.0 have a bug and don’t allow assigning a
        # value to this field, so for compatibility we avoid doing it at runtime.
        __pydantic_extra__: Dict[str, Optional[object]] = FieldInfo(init=False)  # pyright: ignore[reportIncompatibleVariableOverride]

        # Stub to indicate that arbitrary properties are accepted.
        # To access properties that are not valid identifiers you can use `getattr`, e.g.
        # `getattr(obj, '$type')`
        def __getattr__(self, attr: str) -> Optional[object]: ...
    else:
        __pydantic_extra__: Dict[str, Optional[object]]


class Output(BaseModel):
    """JSON Schema for agent input"""

    default: Optional[object] = None
    """Default value for the property"""

    description: Optional[str] = None
    """Description of the schema"""

    enum: Optional[List[Optional[object]]] = None
    """Enumeration of allowed values"""

    items: Optional[object] = None
    """Schema for array items"""

    properties: Optional[Dict[str, Optional[object]]] = None
    """Property definitions for object types"""

    required: Optional[List[str]] = None
    """List of required property names"""

    title: Optional[str] = None
    """Human-readable title"""

    type: Optional[str] = None
    """JSON Schema type (e.g., "object", "string", "array")"""

    if TYPE_CHECKING:
        # Some versions of Pydantic <2.8.0 have a bug and don’t allow assigning a
        # value to this field, so for compatibility we avoid doing it at runtime.
        __pydantic_extra__: Dict[str, Optional[object]] = FieldInfo(init=False)  # pyright: ignore[reportIncompatibleVariableOverride]

        # Stub to indicate that arbitrary properties are accepted.
        # To access properties that are not valid identifiers you can use `getattr`, e.g.
        # `getattr(obj, '$type')`
        def __getattr__(self, attr: str) -> Optional[object]: ...
    else:
        __pydantic_extra__: Dict[str, Optional[object]]


class Tool(BaseModel):
    id: str
    """Unique tool ID"""

    created_at: str = FieldInfo(alias="createdAt")
    """Creation timestamp"""

    created_by: Optional[str] = FieldInfo(alias="createdBy", default=None)
    """User who created the tool (for managed tools)"""

    description: Optional[str] = None
    """Tool description"""

    discovered_at: Optional[str] = FieldInfo(alias="discoveredAt", default=None)
    """When the tool was discovered"""

    input: Optional[Input] = None
    """JSON Schema for agent input"""

    name: str
    """Tool name"""

    output: Optional[Output] = None
    """JSON Schema for agent input"""

    project_id: str = FieldInfo(alias="projectId")
    """Project ID"""

    status: Literal["active", "inactive"]
    """Tool status"""

    type: str
    """
    Tool type: "managed" for platform-provided, or "{language}" for custom tools
    (e.g., "python", "typescript")
    """

    updated_at: str = FieldInfo(alias="updatedAt")
    """Last update timestamp"""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Dict, Optional

import httpx

from ..types import client_token_create_params
from .._types import Body, Omit, Query, Headers, NoneType, NotGiven, omit, not_given
from .._utils import maybe_transform, async_maybe_transform
from .._compat import cached_property
from .._resource import SyncAPIResource, AsyncAPIResource
from .._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from .._base_client import make_request_options
from ..types.client_token_create_response import ClientTokenCreateResponse

__all__ = ["ClientTokensResource", "AsyncClientTokensResource"]


class ClientTokensResource(SyncAPIResource):
    @cached_property
    def with_raw_response(self) -> ClientTokensResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/reminix-ai/reminix-python#accessing-raw-response-data-eg-headers
        """
        return ClientTokensResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> ClientTokensResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/reminix-ai/reminix-python#with_streaming_response
        """
        return ClientTokensResourceWithStreamingResponse(self)

    def create(
        self,
        *,
        context: Dict[str, Optional[object]],
        server_context: Dict[str, Optional[object]] | Omit = omit,
        ttl_seconds: int | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> ClientTokenCreateResponse:
        """
        Create a short-lived client token for browser SDK use.

        **Use case:** Your backend calls this endpoint to generate a token, then passes
        it to your frontend. The frontend uses the token to make authenticated requests
        to `/client/*` endpoints.

        **Context types:**

        - `context`: Public data accessible to the client via `/client/context`
        - `serverContext`: Private data only accessible to agents/handlers (never
          exposed to client)

        **Security:**

        - Tokens are short-lived (default 1 hour, max 24 hours)
        - Both context types are trusted and cannot be tampered with
        - Store the token securely - it will only be shown once

        **Example flow:**

        1. User logs in to your app
        2. Your backend calls `POST /v1/client-tokens` with
           `{ context: { userId: "user_123" }, serverContext: { internalId: "int_456" } }`
        3. Your backend returns the token to your frontend
        4. Frontend uses the token to call `/client/*` endpoints

        Args:
          context:
              Public context accessible to the client via /client/context (e.g., { userId:
              "...", sessionId: "..." })

          server_context: Private context only accessible to agents/handlers, never exposed to client
              (e.g., { internalId: "..." })

          ttl_seconds: Time-to-live in seconds. Default: 3600 (1 hour). Max: 86400 (24 hours).

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return self._post(
            "/client-tokens",
            body=maybe_transform(
                {
                    "context": context,
                    "server_context": server_context,
                    "ttl_seconds": ttl_seconds,
                },
                client_token_create_params.ClientTokenCreateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=ClientTokenCreateResponse,
        )

    def revoke(
        self,
        id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> None:
        """
        Revoke a client token immediately.

        Once revoked, the token can no longer be used for authentication. This is a soft
        delete - the token record is kept for audit purposes.

        Args:
          id: Client token ID to revoke

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        extra_headers = {"Accept": "*/*", **(extra_headers or {})}
        return self._delete(
            f"/client-tokens/{id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=NoneType,
        )


class AsyncClientTokensResource(AsyncAPIResource):
    @cached_property
    def with_raw_response(self) -> AsyncClientTokensResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/reminix-ai/reminix-python#accessing-raw-response-data-eg-headers
        """
        return AsyncClientTokensResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncClientTokensResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/reminix-ai/reminix-python#with_streaming_response
        """
        return AsyncClientTokensResourceWithStreamingResponse(self)

    async def create(
        self,
        *,
        context: Dict[str, Optional[object]],
        server_context: Dict[str, Optional[object]] | Omit = omit,
        ttl_seconds: int | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> ClientTokenCreateResponse:
        """
        Create a short-lived client token for browser SDK use.

        **Use case:** Your backend calls this endpoint to generate a token, then passes
        it to your frontend. The frontend uses the token to make authenticated requests
        to `/client/*` endpoints.

        **Context types:**

        - `context`: Public data accessible to the client via `/client/context`
        - `serverContext`: Private data only accessible to agents/handlers (never
          exposed to client)

        **Security:**

        - Tokens are short-lived (default 1 hour, max 24 hours)
        - Both context types are trusted and cannot be tampered with
        - Store the token securely - it will only be shown once

        **Example flow:**

        1. User logs in to your app
        2. Your backend calls `POST /v1/client-tokens` with
           `{ context: { userId: "user_123" }, serverContext: { internalId: "int_456" } }`
        3. Your backend returns the token to your frontend
        4. Frontend uses the token to call `/client/*` endpoints

        Args:
          context:
              Public context accessible to the client via /client/context (e.g., { userId:
              "...", sessionId: "..." })

          server_context: Private context only accessible to agents/handlers, never exposed to client
              (e.g., { internalId: "..." })

          ttl_seconds: Time-to-live in seconds. Default: 3600 (1 hour). Max: 86400 (24 hours).

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        return await self._post(
            "/client-tokens",
            body=await async_maybe_transform(
                {
                    "context": context,
                    "server_context": server_context,
                    "ttl_seconds": ttl_seconds,
                },
                client_token_create_params.ClientTokenCreateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=ClientTokenCreateResponse,
        )

    async def revoke(
        self,
        id: str,
        *,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> None:
        """
        Revoke a client token immediately.

        Once revoked, the token can no longer be used for authentication. This is a soft
        delete - the token record is kept for audit purposes.

        Args:
          id: Client token ID to revoke

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not id:
            raise ValueError(f"Expected a non-empty value for `id` but received {id!r}")
        extra_headers = {"Accept": "*/*", **(extra_headers or {})}
        return await self._delete(
            f"/client-tokens/{id}",
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=NoneType,
        )


class ClientTokensResourceWithRawResponse:
    def __init__(self, client_tokens: ClientTokensResource) -> None:
        self._client_tokens = client_tokens

        self.create = to_raw_response_wrapper(
            client_tokens.create,
        )
        self.revoke = to_raw_response_wrapper(
            client_tokens.revoke,
        )


class AsyncClientTokensResourceWithRawResponse:
    def __init__(self, client_tokens: AsyncClientTokensResource) -> None:
        self._client_tokens = client_tokens

        self.create = async_to_raw_response_wrapper(
            client_tokens.create,
        )
        self.revoke = async_to_raw_response_wrapper(
            client_tokens.revoke,
        )


class ClientTokensResourceWithStreamingResponse:
    def __init__(self, client_tokens: ClientTokensResource) -> None:
        self._client_tokens = client_tokens

        self.create = to_streamed_response_wrapper(
            client_tokens.create,
        )
        self.revoke = to_streamed_response_wrapper(
            client_tokens.revoke,
        )


class AsyncClientTokensResourceWithStreamingResponse:
    def __init__(self, client_tokens: AsyncClientTokensResource) -> None:
        self._client_tokens = client_tokens

        self.create = async_to_streamed_response_wrapper(
            client_tokens.create,
        )
        self.revoke = async_to_streamed_response_wrapper(
            client_tokens.revoke,
        )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

import os
from typing import Any, cast

import pytest

from reminix import Reminix, AsyncReminix
from tests.utils import assert_matches_type
from reminix.types import (
    Agent,
    AgentChatResponse,
    AgentInvokeResponse,
)
from reminix.pagination import SyncCursor, AsyncCursor

base_url = os.environ.get("TEST_API_BASE_URL", "http://127.0.0.1:4010")


class TestAgents:
    parametrize = pytest.mark.parametrize("client", [False, True], indirect=True, ids=["loose", "strict"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_method_retrieve(self, client: Reminix) -> None:
        agent = client.agents.retrieve(
            "x",
        )
        assert_matches_type(Agent, agent, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_raw_response_retrieve(self, client: Reminix) -> None:
        response = client.agents.with_raw_response.retrieve(
            "x",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        agent = response.parse()
        assert_matches_type(Agent, agent, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_streaming_response_retrieve(self, client: Reminix) -> None:
        with client.agents.with_streaming_response.retrieve(
            "x",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            agent = response.parse()
            assert_matches_type(Agent, agent, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_path_params_retrieve(self, client: Reminix) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `name` but received ''"):
            client.agents.with_raw_response.retrieve(
                "",
            )

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_method_list(self, client: Reminix) -> None:
        agent = client.agents.list()
        assert_matches_type(SyncCursor[Agent], agent, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_method_list_with_all_params(self, client: Reminix) -> None:
        agent = client.agents.list(
            cursor="cursor",
            limit=1,
            status="active",
            type="type",
        )
        assert_matches_type(SyncCursor[Agent], agent, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_raw_response_list(self, client: Reminix) -> None:
        response = client.agents.with_raw_response.list()

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        agent = response.parse()
        assert_matches_type(SyncCursor[Agent], agent, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_streaming_response_list(self, client: Reminix) -> None:
        with client.agents.with_streaming_response.list() as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            agent = response.parse()
            assert_matches_type(SyncCursor[Agent], agent, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_method_chat_overload_1(self, client: Reminix) -> None:
        agent = client.agents.chat(
            name="name",
            messages=[
                {
                    "content": "string",
                    "role": "system",
                }
            ],
        )
        assert_matches_type(AgentChatResponse, agent, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_method_chat_with_all_params_overload_1(self, client: Reminix) -> None:
        agent = client.agents.chat(
            name="name",
            messages=[
                {
                    "content": "string",
                    "role": "system",
                    "name": "name",
                    "tool_call_id": "tool_call_id",
                    "tool_calls": [
                        {
                            "id": "id",
                            "function": {
                                "arguments": "arguments",
                                "name": "name",
                            },
                            "type": "function",
                        }
                    ],
                }
            ],
            context={"identity": {"foo": "bar"}},
            conversation_id="conversation_id",
            stream=False,
        )
        assert_matches_type(AgentChatResponse, agent, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_raw_response_chat_overload_1(self, client: Reminix) -> None:
        response = client.agents.with_raw_response.chat(
            name="name",
            messages=[
                {
                    "content": "string",
                    "role": "system",
                }
            ],
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        agent = response.parse()
        assert_matches_type(AgentChatResponse, agent, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_streaming_response_chat_overload_1(self, client: Reminix) -> None:
        with client.agents.with_streaming_response.chat(
            name="name",
            messages=[
                {
                    "content": "string",
                    "role": "system",
                }
            ],
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            agent = response.parse()
            assert_matches_type(AgentChatResponse, agent, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_path_params_chat_overload_1(self, client: Reminix) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `name` but received ''"):
            client.agents.with_raw_response.chat(
                name="",
                messages=[
                    {
                        "content": "string",
                        "role": "system",
                    }
                ],
            )

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_method_chat_overload_2(self, client: Reminix) -> None:
        agent_stream = client.agents.chat(
            name="name",
            messages=[
                {
                    "content": "string",
                    "role": "system",
                }
            ],
            stream=True,
        )
        agent_stream.response.close()

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_method_chat_with_all_params_overload_2(self, client: Reminix) -> None:
        agent_stream = client.agents.chat(
            name="name",
            messages=[
                {
                    "content": "string",
                    "role": "system",
                    "name": "name",
                    "tool_call_id": "tool_call_id",
                    "tool_calls": [
                        {
                            "id": "id",
                            "function": {
                                "arguments": "arguments",
                                "name": "name",
                            },
                            "type": "function",
                        }
                    ],
                }
            ],
            stream=True,
            context={"identity": {"foo": "bar"}},
            conversation_id="conversation_id",
        )
        agent_stream.response.close()

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_raw_response_chat_overload_2(self, client: Reminix) -> None:
        response = client.agents.with_raw_response.chat(
            name="name",
            messages=[
                {
                    "content": "string",
                    "role": "system",
                }
            ],
            stream=True,
        )

        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        stream = response.parse()
        stream.close()

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_streaming_response_chat_overload_2(self, client: Reminix) -> None:
        with client.agents.with_streaming_response.chat(
            name="name",
            messages=[
                {
                    "content": "string",
                    "role": "system",
                }
            ],
            stream=True,
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            stream = response.parse()
            stream.close()

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_path_params_chat_overload_2(self, client: Reminix) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `name` but received ''"):
            client.agents.with_raw_response.chat(
                name="",
                messages=[
                    {
                        "content": "string",
                        "role": "system",
                    }
                ],
                stream=True,
            )

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_method_invoke_overload_1(self, client: Reminix) -> None:
        agent = client.agents.invoke(
            name="name",
        )
        assert_matches_type(AgentInvokeResponse, agent, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_method_invoke_with_all_params_overload_1(self, client: Reminix) -> None:
        agent = client.agents.invoke(
            name="name",
            context={"identity": {"foo": "bar"}},
            input={"prompt": "bar"},
            stream=False,
        )
        assert_matches_type(AgentInvokeResponse, agent, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_raw_response_invoke_overload_1(self, client: Reminix) -> None:
        response = client.agents.with_raw_response.invoke(
            name="name",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        agent = response.parse()
        assert_matches_type(AgentInvokeResponse, agent, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_streaming_response_invoke_overload_1(self, client: Reminix) -> None:
        with client.agents.with_streaming_response.invoke(
            name="name",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            agent = response.parse()
            assert_matches_type(AgentInvokeResponse, agent, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_path_params_invoke_overload_1(self, client: Reminix) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `name` but received ''"):
            client.agents.with_raw_response.invoke(
                name="",
            )

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_method_invoke_overload_2(self, client: Reminix) -> None:
        agent_stream = client.agents.invoke(
            name="name",
            stream=True,
        )
        agent_stream.response.close()

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_method_invoke_with_all_params_overload_2(self, client: Reminix) -> None:
        agent_stream = client.agents.invoke(
            name="name",
            stream=True,
            context={"identity": {"foo": "bar"}},
            input={"prompt": "bar"},
        )
        agent_stream.response.close()

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_raw_response_invoke_overload_2(self, client: Reminix) -> None:
        response = client.agents.with_raw_response.invoke(
            name="name",
            stream=True,
        )

        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        stream = response.parse()
        stream.close()

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_streaming_response_invoke_overload_2(self, client: Reminix) -> None:
        with client.agents.with_streaming_response.invoke(
            name="name",
            stream=True,
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            stream = response.parse()
            stream.close()

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_path_params_invoke_overload_2(self, client: Reminix) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `name` but received ''"):
            client.agents.with_raw_response.invoke(
                name="",
                stream=True,
            )


class TestAsyncAgents:
    parametrize = pytest.mark.parametrize(
        "async_client", [False, True, {"http_client": "aiohttp"}], indirect=True, ids=["loose", "strict", "aiohttp"]
    )

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_method_retrieve(self, async_client: AsyncReminix) -> None:
        agent = await async_client.agents.retrieve(
            "x",
        )
        assert_matches_type(Agent, agent, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_raw_response_retrieve(self, async_client: AsyncReminix) -> None:
        response = await async_client.agents.with_raw_response.retrieve(
            "x",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        agent = await response.parse()
        assert_matches_type(Agent, agent, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_streaming_response_retrieve(self, async_client: AsyncReminix) -> None:
        async with async_client.agents.with_streaming_response.retrieve(
            "x",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            agent = await response.parse()
            assert_matches_type(Agent, agent, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_path_params_retrieve(self, async_client: AsyncReminix) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `name` but received ''"):
            await async_client.agents.with_raw_response.retrieve(
                "",
            )

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_method_list(self, async_client: AsyncReminix) -> None:
        agent = await async_client.agents.list()
        assert_matches_type(AsyncCursor[Agent], agent, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_method_list_with_all_params(self, async_client: AsyncReminix) -> None:
        agent = await async_client.agents.list(
            cursor="cursor",
            limit=1,
            status="active",
            type="type",
        )
        assert_matches_type(AsyncCursor[Agent], agent, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_raw_response_list(self, async_client: AsyncReminix) -> None:
        response = await async_client.agents.with_raw_response.list()

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        agent = await response.parse()
        assert_matches_type(AsyncCursor[Agent], agent, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_streaming_response_list(self, async_client: AsyncReminix) -> None:
        async with async_client.agents.with_streaming_response.list() as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            agent = await response.parse()
            assert_matches_type(AsyncCursor[Agent], agent, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_method_chat_overload_1(self, async_client: AsyncReminix) -> None:
        agent = await async_client.agents.chat(
            name="name",
            messages=[
                {
                    "content": "string",
                    "role": "system",
                }
            ],
        )
        assert_matches_type(AgentChatResponse, agent, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_method_chat_with_all_params_overload_1(self, async_client: AsyncReminix) -> None:
        agent = await async_client.agents.chat(
            name="name",
            messages=[
                {
                    "content": "string",
                    "role": "system",
                    "name": "name",
                    "tool_call_id": "tool_call_id",
                    "tool_calls": [
                        {
                            "id": "id",
                            "function": {
                                "arguments": "arguments",
                                "name": "name",
                            },
                            "type": "function",
                        }
                    ],
                }
            ],
            context={"identity": {"foo": "bar"}},
            conversation_id="conversation_id",
            stream=False,
        )
        assert_matches_type(AgentChatResponse, agent, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_raw_response_chat_overload_1(self, async_client: AsyncReminix) -> None:
        response = await async_client.agents.with_raw_response.chat(
            name="name",
            messages=[
                {
                    "content": "string",
                    "role": "system",
                }
            ],
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        agent = await response.parse()
        assert_matches_type(AgentChatResponse, agent, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_streaming_response_chat_overload_1(self, async_client: AsyncReminix) -> None:
        async with async_client.agents.with_streaming_response.chat(
            name="name",
            messages=[
                {
                    "content": "string",
                    "role": "system",
                }
            ],
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            agent = await response.parse()
            assert_matches_type(AgentChatResponse, agent, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_path_params_chat_overload_1(self, async_client: AsyncReminix) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `name` but received ''"):
            await async_client.agents.with_raw_response.chat(
                name="",
                messages=[
                    {
                        "content": "string",
                        "role": "system",
                    }
                ],
            )

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_method_chat_overload_2(self, async_client: AsyncReminix) -> None:
        agent_stream = await async_client.agents.chat(
            name="name",
            messages=[
                {
                    "content": "string",
                    "role": "system",
                }
            ],
            stream=True,
        )
        await agent_stream.response.aclose()

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_method_chat_with_all_params_overload_2(self, async_client: AsyncReminix) -> None:
        agent_stream = await async_client.agents.chat(
            name="name",
            messages=[
                {
                    "content": "string",
                    "role": "system",
                    "name": "name",
                    "tool_call_id": "tool_call_id",
                    "tool_calls": [
                        {
                            "id": "id",
                            "function": {
                                "arguments": "arguments",
                                "name": "name",
                            },
                            "type": "function",
                        }
                    ],
                }
            ],
            stream=True,
            context={"identity": {"foo": "bar"}},
            conversation_id="conversation_id",
        )
        await agent_stream.response.aclose()

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_raw_response_chat_overload_2(self, async_client: AsyncReminix) -> None:
        response = await async_client.agents.with_raw_response.chat(
            name="name",
            messages=[
                {
                    "content": "string",
                    "role": "system",
                }
            ],
            stream=True,
        )

        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        stream = await response.parse()
        await stream.close()

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_streaming_response_chat_overload_2(self, async_client: AsyncReminix) -> None:
        async with async_client.agents.with_streaming_response.chat(
            name="name",
            messages=[
                {
                    "content": "string",
                    "role": "system",
                }
            ],
            stream=True,
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            stream = await response.parse()
            await stream.close()

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_path_params_chat_overload_2(self, async_client: AsyncReminix) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `name` but received ''"):
            await async_client.agents.with_raw_response.chat(
                name="",
                messages=[
                    {
                        "content": "string",
                        "role": "system",
                    }
                ],
                stream=True,
            )

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_method_invoke_overload_1(self, async_client: AsyncReminix) -> None:
        agent = await async_client.agents.invoke(
            name="name",
        )
        assert_matches_type(AgentInvokeResponse, agent, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_method_invoke_with_all_params_overload_1(self, async_client: AsyncReminix) -> None:
        agent = await async_client.agents.invoke(
            name="name",
            context={"identity": {"foo": "bar"}},
            input={"prompt": "bar"},
            stream=False,
        )
        assert_matches_type(AgentInvokeResponse, agent, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_raw_response_invoke_overload_1(self, async_client: AsyncReminix) -> None:
        response = await async_client.agents.with_raw_response.invoke(
            name="name",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        agent = await response.parse()
        assert_matches_type(AgentInvokeResponse, agent, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_streaming_response_invoke_overload_1(self, async_client: AsyncReminix) -> None:
        async with async_client.agents.with_streaming_response.invoke(
            name="name",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            agent = await response.parse()
            assert_matches_type(AgentInvokeResponse, agent, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_path_params_invoke_overload_1(self, async_client: AsyncReminix) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `name` but received ''"):
            await async_client.agents.with_raw_response.invoke(
                name="",
            )

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_method_invoke_overload_2(self, async_client: AsyncReminix) -> None:
        agent_stream = await async_client.agents.invoke(
            name="name",
            stream=True,
        )
        await agent_stream.response.aclose()

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_method_invoke_with_all_params_overload_2(self, async_client: AsyncReminix) -> None:
        agent_stream = await async_client.agents.invoke(
            name="name",
            stream=True,
            context={"identity": {"foo": "bar"}},
            input={"prompt": "bar"},
        )
        await agent_stream.response.aclose()

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_raw_response_invoke_overload_2(self, async_client: AsyncReminix) -> None:
        response = await async_client.agents.with_raw_response.invoke(
            name="name",
            stream=True,
        )

        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        stream = await response.parse()
        await stream.close()

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_streaming_response_invoke_overload_2(self, async_client: AsyncReminix) -> None:
        async with async_client.agents.with_streaming_response.invoke(
            name="name",
            stream=True,
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            stream = await response.parse()
            await stream.close()

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_path_params_invoke_overload_2(self, async_client: AsyncReminix) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `name` but received ''"):
            await async_client.agents.with_raw_response.invoke(
                name="",
                stream=True,
            )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Dict, List, Union, Optional
from typing_extensions import Literal

from .._models import BaseModel

__all__ = [
    "ChatMessage",
    "ContentMultimodalContent",
    "ContentMultimodalContentImageURL",
    "ToolCall",
    "ToolCallFunction",
]


class ContentMultimodalContentImageURL(BaseModel):
    url: str


class ContentMultimodalContent(BaseModel):
    type: Literal["text", "image_url"]

    image_url: Optional[ContentMultimodalContentImageURL] = None

    text: Optional[str] = None


class ToolCallFunction(BaseModel):
    arguments: str

    name: str


class ToolCall(BaseModel):
    id: str

    function: ToolCallFunction

    type: Literal["function"]


class ChatMessage(BaseModel):
    content: Union[str, List[ContentMultimodalContent], Dict[str, Optional[object]], None] = None
    """Message content.

    Can be string, array (multimodal), object (tool), or null (when tool_calls
    present).
    """

    role: Literal["system", "user", "assistant", "tool"]
    """Message role"""

    name: Optional[str] = None
    """Tool name (required when role is "tool")"""

    tool_call_id: Optional[str] = None
    """Tool call ID (for tool role messages)"""

    tool_calls: Optional[List[ToolCall]] = None
    """Tool calls requested by assistant (for assistant role messages)"""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

import os
from typing import Any, cast

import pytest

from reminix import Reminix, AsyncReminix
from tests.utils import assert_matches_type
from reminix.types import ClientTokenCreateResponse

base_url = os.environ.get("TEST_API_BASE_URL", "http://127.0.0.1:4010")


class TestClientTokens:
    parametrize = pytest.mark.parametrize("client", [False, True], indirect=True, ids=["loose", "strict"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_method_create(self, client: Reminix) -> None:
        client_token = client.client_tokens.create(
            context={
                "userId": "bar",
                "sessionId": "bar",
            },
        )
        assert_matches_type(ClientTokenCreateResponse, client_token, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_method_create_with_all_params(self, client: Reminix) -> None:
        client_token = client.client_tokens.create(
            context={
                "userId": "bar",
                "sessionId": "bar",
            },
            server_context={"foo": "bar"},
            ttl_seconds=1,
        )
        assert_matches_type(ClientTokenCreateResponse, client_token, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_raw_response_create(self, client: Reminix) -> None:
        response = client.client_tokens.with_raw_response.create(
            context={
                "userId": "bar",
                "sessionId": "bar",
            },
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        client_token = response.parse()
        assert_matches_type(ClientTokenCreateResponse, client_token, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_streaming_response_create(self, client: Reminix) -> None:
        with client.client_tokens.with_streaming_response.create(
            context={
                "userId": "bar",
                "sessionId": "bar",
            },
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            client_token = response.parse()
            assert_matches_type(ClientTokenCreateResponse, client_token, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_method_revoke(self, client: Reminix) -> None:
        client_token = client.client_tokens.revoke(
            "x",
        )
        assert client_token is None

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_raw_response_revoke(self, client: Reminix) -> None:
        response = client.client_tokens.with_raw_response.revoke(
            "x",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        client_token = response.parse()
        assert client_token is None

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_streaming_response_revoke(self, client: Reminix) -> None:
        with client.client_tokens.with_streaming_response.revoke(
            "x",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            client_token = response.parse()
            assert client_token is None

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    def test_path_params_revoke(self, client: Reminix) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `id` but received ''"):
            client.client_tokens.with_raw_response.revoke(
                "",
            )


class TestAsyncClientTokens:
    parametrize = pytest.mark.parametrize(
        "async_client", [False, True, {"http_client": "aiohttp"}], indirect=True, ids=["loose", "strict", "aiohttp"]
    )

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_method_create(self, async_client: AsyncReminix) -> None:
        client_token = await async_client.client_tokens.create(
            context={
                "userId": "bar",
                "sessionId": "bar",
            },
        )
        assert_matches_type(ClientTokenCreateResponse, client_token, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_method_create_with_all_params(self, async_client: AsyncReminix) -> None:
        client_token = await async_client.client_tokens.create(
            context={
                "userId": "bar",
                "sessionId": "bar",
            },
            server_context={"foo": "bar"},
            ttl_seconds=1,
        )
        assert_matches_type(ClientTokenCreateResponse, client_token, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_raw_response_create(self, async_client: AsyncReminix) -> None:
        response = await async_client.client_tokens.with_raw_response.create(
            context={
                "userId": "bar",
                "sessionId": "bar",
            },
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        client_token = await response.parse()
        assert_matches_type(ClientTokenCreateResponse, client_token, path=["response"])

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_streaming_response_create(self, async_client: AsyncReminix) -> None:
        async with async_client.client_tokens.with_streaming_response.create(
            context={
                "userId": "bar",
                "sessionId": "bar",
            },
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            client_token = await response.parse()
            assert_matches_type(ClientTokenCreateResponse, client_token, path=["response"])

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_method_revoke(self, async_client: AsyncReminix) -> None:
        client_token = await async_client.client_tokens.revoke(
            "x",
        )
        assert client_token is None

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_raw_response_revoke(self, async_client: AsyncReminix) -> None:
        response = await async_client.client_tokens.with_raw_response.revoke(
            "x",
        )

        assert response.is_closed is True
        assert response.http_request.headers.get("X-Stainless-Lang") == "python"
        client_token = await response.parse()
        assert client_token is None

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_streaming_response_revoke(self, async_client: AsyncReminix) -> None:
        async with async_client.client_tokens.with_streaming_response.revoke(
            "x",
        ) as response:
            assert not response.is_closed
            assert response.http_request.headers.get("X-Stainless-Lang") == "python"

            client_token = await response.parse()
            assert client_token is None

        assert cast(Any, response.is_closed) is True

    @pytest.mark.skip(reason="Prism tests are disabled")
    @parametrize
    async def test_path_params_revoke(self, async_client: AsyncReminix) -> None:
        with pytest.raises(ValueError, match=r"Expected a non-empty value for `id` but received ''"):
            await async_client.client_tokens.with_raw_response.revoke(
                "",
            )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import List

from pydantic import Field as FieldInfo

from .._models import BaseModel

__all__ = ["KnowledgeSearchResponse", "Chunk"]


class Chunk(BaseModel):
    chunk_index: float = FieldInfo(alias="chunkIndex")
    """Position in document"""

    content: str
    """Chunk text content"""

    document_id: str = FieldInfo(alias="documentId")
    """Source document ID"""

    score: float
    """Similarity score (0-1)"""


class KnowledgeSearchResponse(BaseModel):
    chunks: List[Chunk]

    count: float
    """Number of results found"""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Dict, Union, Iterable, Optional
from typing_extensions import Literal, Required, TypedDict

__all__ = [
    "ChatMessageParam",
    "ContentMultimodalContent",
    "ContentMultimodalContentImageURL",
    "ToolCall",
    "ToolCallFunction",
]


class ContentMultimodalContentImageURL(TypedDict, total=False):
    url: Required[str]


class ContentMultimodalContent(TypedDict, total=False):
    type: Required[Literal["text", "image_url"]]

    image_url: ContentMultimodalContentImageURL

    text: str


class ToolCallFunction(TypedDict, total=False):
    arguments: Required[str]

    name: Required[str]


class ToolCall(TypedDict, total=False):
    id: Required[str]

    function: Required[ToolCallFunction]

    type: Required[Literal["function"]]


class ChatMessageParam(TypedDict, total=False):
    content: Required[Union[str, Iterable[ContentMultimodalContent], Dict[str, Optional[object]], None]]
    """Message content.

    Can be string, array (multimodal), object (tool), or null (when tool_calls
    present).
    """

    role: Required[Literal["system", "user", "assistant", "tool"]]
    """Message role"""

    name: str
    """Tool name (required when role is "tool")"""

    tool_call_id: str
    """Tool call ID (for tool role messages)"""

    tool_calls: Iterable[ToolCall]
    """Tool calls requested by assistant (for assistant role messages)"""
